@@ -48,7 +48,6 @@
  */
 static void deallocate_inode(e2fsck_t ctx, ino_t ino,
 			     char* block_buf);
-static int process_bad_inode(e2fsck_t ctx, ino_t dir, ino_t ino);
 static int check_dir_block(ext2_filsys fs,
 			   struct ext2_db_entry *dir_blocks_info,
 			   void *priv_data);
@@ -136,8 +135,10 @@ void e2fsck_pass2(e2fsck_t ctx)
 		ctx->inode_bad_map = 0;
 	}
 #ifdef RESOURCE_TRACK
-	if (ctx->options & E2F_OPT_TIME2)
+	if (ctx->options & E2F_OPT_TIME2) {
+		e2fsck_clear_progbar(ctx);
 		print_resource_track("Pass 2", &rtrack);
+	}
 #endif
 }
 
@@ -451,7 +452,8 @@ static int check_dir_block(ext2_filsys fs,
 		if (ctx->inode_bad_map &&
 		    ext2fs_test_inode_bitmap(ctx->inode_bad_map,
 					     dirent->inode)) {
-			if (process_bad_inode(ctx, ino, dirent->inode)) {
+			if (e2fsck_process_bad_inode(ctx, ino,
+						     dirent->inode)) {
 				dirent->inode = 0;
 				dir_modified++;
 				goto next;
@@ -585,7 +587,7 @@ static void deallocate_inode(e2fsck_t ctx, ino_t ino,
 	}
 }
 
-static int process_bad_inode(e2fsck_t ctx, ino_t dir, ino_t ino)
+extern int e2fsck_process_bad_inode(e2fsck_t ctx, ino_t dir, ino_t ino)
 {
 	ext2_filsys fs = ctx->fs;
 	struct ext2_inode	inode;
