@@ -8,7 +8,7 @@
  *
  * Written by Theodore Ts'o, <tytso@mit.edu>
  * 
- * Usage:	fsck [-AVRNTM] [-s] [-t fstype] [fs-options] device
+ * Usage:	fsck [-ACVRNTM] [-s] [-t fstype] [fs-options] device
  * 
  * Miquel van Smoorenburg (miquels@drinkel.ow.org) 20-Oct-1994:
  *   o Changed -t fstype to behave like with mount when -A (all file
@@ -18,7 +18,7 @@
  *     can be added without changing this front-end.
  *   o -R flag skip root file system.
  *
- * Copyright (C) 1993, 1994, 1995, 1996, 1997 Theodore Ts'o.
+ * Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999 Theodore Ts'o.
  *
  * %Begin-Header%
  * This file may be redistributed under the terms of the GNU Public
@@ -52,6 +52,7 @@
 
 #include "../version.h"
 #include "fsck.h"
+#include "get_device_by_label.h"
 
 #ifndef _PATH_MNTTAB
 #define	_PATH_MNTTAB	"/etc/fstab"
@@ -129,6 +130,7 @@ int skip_root = 0;
 int like_mount = 0;
 int notitle = 0;
 int parallel_root = 0;
+int progress = 0;
 char *progname;
 char *fstype = NULL;
 struct fs_info *filesys_info;
@@ -241,6 +243,24 @@ int parse_fstab_line(char *line, struct fs_info **ret_fs)
 	return 0;
 }
 
+/*
+ * Interpret the device name if necessary 
+ */
+static char *interpret_device(char *spec)
+{
+	char *dev = NULL;
+	
+	if (!strncmp(spec, "UUID=", 5))
+		dev = get_spec_by_uuid(spec+5);
+	else if (!strncmp(spec, "LABEL=", 6))
+		dev = get_spec_by_volume_label(spec+6);
+	if (dev) {
+		free(spec);
+		spec = dev;
+	}
+	return spec;
+}
+
 /*
  * Load the filesystem database from /etc/fstab
  */
@@ -270,6 +290,7 @@ static void load_fs_info(char *filename)
 		}
 		if (!fs)
 			continue;
+		fs->device = interpret_device(fs->device);
 		if (!filesys_info)
 			filesys_info = fs;
 		else
@@ -335,23 +356,49 @@ static char *find_fsck(char *type)
   return(s ? prog : NULL);
 }
 
+static int progress_active()
+{
+	struct fsck_instance *inst;
+
+	for (inst = instance_list; inst; inst = inst->next) {
+		if (inst->flags & FLAG_DONE)
+			continue;
+		if (inst->flags & FLAG_PROGRESS)
+			return 1;
+	}
+	return 0;
+}
+
 /*
  * Execute a particular fsck program, and link it into the list of
  * child processes we are waiting for.
  */
-static int execute(char *prog, char *device, char *mntpt)
+static int execute(char *type, char *device, char *mntpt, int interactive)
 {
-	char *s, *argv[80];
+	char *s, *argv[80], prog[80];
 	int  argc, i;
 	struct fsck_instance *inst;
 	pid_t	pid;
 
+	inst = malloc(sizeof(struct fsck_instance));
+	if (!inst)
+		return ENOMEM;
+	memset(inst, 0, sizeof(struct fsck_instance));
+
+	sprintf(prog, "fsck.%s", type);
 	argv[0] = string_copy(prog);
 	argc = 1;
 	
 	for (i=0; i <num_args; i++)
 		argv[argc++] = string_copy(args[i]);
 
+	if (progress & !progress_active()) {
+		if (strcmp(type, "ext2") == 0) {
+			argv[argc++] = "-C0";
+			inst->flags |= FLAG_PROGRESS;
+		}
+	}
+
 	argv[argc++] = string_copy(device);
 	argv[argc] = 0;
 
@@ -362,29 +409,29 @@ static int execute(char *prog, char *device, char *mntpt)
 	}
 
 	if (verbose || noexecute) {
-		printf("[%s -- %s] ", s, mntpt);
+		printf("[%s -- %s] ", s, mntpt ? mntpt : device);
 		for (i=0; i < argc; i++)
 			printf("%s ", argv[i]);
 		printf("\n");
 	}
-	if (noexecute)
-		return 0;
 	
 	/* Fork and execute the correct program. */
-	if ((pid = fork()) < 0) {
+	if (noexecute)
+		pid = -1;
+	else if ((pid = fork()) < 0) {
 		perror("fork");
 		return errno;
 	} else if (pid == 0) {
+		if (!interactive)
+			close(0);
 		(void) execv(s, argv);
 		perror(argv[0]);
 		exit(EXIT_ERROR);
 	}
-	inst = malloc(sizeof(struct fsck_instance));
-	if (!inst)
-		return ENOMEM;
-	memset(inst, 0, sizeof(struct fsck_instance));
+
 	inst->pid = pid;
 	inst->prog = string_copy(prog);
+	inst->type = string_copy(type);
 	inst->device = string_copy(device);
 	inst->next = instance_list;
 	instance_list = inst;
@@ -400,12 +447,19 @@ static struct fsck_instance *wait_one(NOARGS)
 {
 	int	status;
 	int	sig;
-	struct fsck_instance *inst, *prev;
+	struct fsck_instance *inst, *inst2, *prev;
 	pid_t	pid;
 
 	if (!instance_list)
 		return NULL;
 
+	if (noexecute) {
+		inst = instance_list;
+		instance_list = inst->next;
+		inst->exit_status = 0;
+		return(inst);
+	}
+
 retry:
 	pid = wait(&status);
 	if (pid < 0) {
@@ -453,6 +507,17 @@ retry:
 		prev->next = inst->next;
 	else
 		instance_list = inst->next;
+	if (progress && (inst->flags & FLAG_PROGRESS) &&
+	    !progress_active()) {
+		for (inst2 = instance_list; inst2; inst2 = inst2->next) {
+			if (inst2->flags & FLAG_DONE)
+				continue;
+			if (strcmp(inst2->type, "ext2"))
+				continue;
+			kill(inst2->pid, SIGUSR1);
+			break;
+		}
+	}
 	return inst;
 }
 
@@ -485,12 +550,11 @@ static int wait_all(NOARGS)
  * If the type isn't specified by the user, then use either the type
  * specified in /etc/fstab, or DEFAULT_FSTYPE.
  */
-static void fsck_device(char *device)
+static void fsck_device(char *device, int interactive)
 {
-	const char	*type = 0;
+	char	*type = 0;
 	struct fs_info *fsent;
 	int retval;
-	char prog[80];
 
 	if (fstype && strncmp(fstype, "no", 2) && !strchr(fstype, ','))
 		type = fstype;
@@ -503,11 +567,11 @@ static void fsck_device(char *device)
 	if (!type)
 		type = DEFAULT_FSTYPE;
 
-	sprintf(prog, "fsck.%s", type);
-	retval = execute(prog, device, fsent->mountpt);
+	retval = execute(type, device, fsent ? fsent->mountpt : 0,
+			 interactive);
 	if (retval) {
-		fprintf(stderr, "%s: Error %d while executing %s for %s\n",
-			progname, retval, prog, device);
+		fprintf(stderr, "%s: Error %d while executing fsck.%s "
+			"for %s\n", progname, retval, type, device);
 	}
 }
 
@@ -602,15 +666,12 @@ static const char *base_device(char *device)
 static int device_already_active(char *device)
 {
 	struct fsck_instance *inst;
-	const char *base;
-
-	base = base_device(device);
+	const char *base = base_device(device);
 
 	for (inst = instance_list; inst; inst = inst->next) {
 		if (!strcmp(base, base_device(inst->device)))
 			return 1;
 	}
-
 	return 0;
 }
 
@@ -621,7 +682,7 @@ static int check_all(NOARGS)
 	struct fsck_instance *inst;
 	int status = EXIT_OK;
 	int not_done_yet = 1;
-	int passno = 0;
+	int passno = 1;
 	int pass_done;
 
 	if (verbose)
@@ -636,7 +697,7 @@ static int check_all(NOARGS)
 				break;
 		}
 		if (fs && !skip_root && !ignore(fs)) {
-			fsck_device(fs->device);
+			fsck_device(fs->device, 0);
 			fs->flags |= FLAG_DONE;
 			status |= wait_all();
 			if (status > EXIT_NONDESTRUCT)
@@ -681,7 +742,7 @@ static int check_all(NOARGS)
 			/*
 			 * Spawn off the fsck process
 			 */
-			fsck_device(fs->device);
+			fsck_device(fs->device, 0);
 			fs->flags |= FLAG_DONE;
 
 			if (serialize) {
@@ -690,6 +751,8 @@ static int check_all(NOARGS)
 
 			}
 		}
+		if (verbose > 1)
+			printf("--waiting-- (pass %d)\n", passno);
 		inst = wait_one();
 		if (inst) {
 			status |= inst->exit_status;
@@ -697,7 +760,7 @@ static int check_all(NOARGS)
 		}
 		if (pass_done) {
 			status |= wait_all();
-			if (verbose) 
+			if (verbose > 1) 
 				printf("----------------------------------\n");
 			passno++;
 		} else
@@ -710,7 +773,7 @@ static int check_all(NOARGS)
 static void usage(NOARGS)
 {
 	fprintf(stderr,
-		"Usage: fsck [-AV] [-t fstype] [fs-options] filesys\n");
+		"Usage: fsck [-ACNPRTV] [-t fstype] [fs-options] filesys\n");
 	exit(EXIT_USAGE);
 }
 
@@ -732,16 +795,19 @@ static void PRS(int argc, char *argv[])
 		arg = argv[i];
 		if (!arg)
 			continue;
-		if (arg[0] == '/') {
+		if ((arg[0] == '/' && !opts_for_fsck) ||
+		    (strncmp(arg, "LABEL=", 6) == 0) ||
+		    (strncmp(arg, "UUID=", 5) == 0)) {
 			if (num_devices >= MAX_DEVICES) {
 				fprintf(stderr, "%s: too many devices\n",
 					progname);
 				exit(1);
 			}
-			devices[num_devices++] = string_copy(arg);
+			devices[num_devices++] =
+				interpret_device(string_copy(arg));
 			continue;
 		}
-		if (arg[0] != '-') {
+		if (arg[0] != '-' || opts_for_fsck) {
 			if (num_args >= MAX_ARGS) {
 				fprintf(stderr, "%s: too many arguments\n",
 					progname);
@@ -759,6 +825,9 @@ static void PRS(int argc, char *argv[])
 			case 'A':
 				doall++;
 				break;
+			case 'C':
+				progress++;
+				break;
 			case 'V':
 				verbose++;
 				break;
@@ -820,7 +889,9 @@ int main(int argc, char *argv[])
 {
 	int i;
 	int status = 0;
+	int interactive = 0;
 	char *oldpath = getenv("PATH");
+	char *fstab;
 
 	PRS(argc, argv);
 
@@ -828,7 +899,10 @@ int main(int argc, char *argv[])
 		printf("Parallelizing fsck version %s (%s)\n",
 			E2FSPROGS_VERSION, E2FSPROGS_DATE);
 
-	load_fs_info(_PATH_MNTTAB);
+	fstab = getenv("FSTAB_FILE");
+	if (!fstab)
+		fstab = _PATH_MNTTAB;
+	load_fs_info(fstab);
 
 	/* Update our search path to include uncommon directories. */
 	if (oldpath) {
@@ -845,8 +919,11 @@ int main(int argc, char *argv[])
 	if (doall)
 		return check_all();
 
+	if ((num_devices == 1) || (serialize))
+		interactive = 1;
+
 	for (i = 0 ; i < num_devices; i++) {
-		fsck_device(devices[i]);
+		fsck_device(devices[i], interactive);
 		if (serialize) {
 			struct fsck_instance *inst;
 
